@@ -1,5 +1,5 @@
 /* remote-config.c -- 
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2008, 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -74,7 +74,6 @@ static int format_parser(struct nv_pair *nv, int line,
 		remote_conf_t *config);
 static int heartbeat_timeout_parser(struct nv_pair *nv, int line, 
 		remote_conf_t *config);
-#ifdef USE_GSSAPI
 static int enable_krb5_parser(struct nv_pair *nv, int line, 
 		remote_conf_t *config);
 static int krb5_principal_parser(struct nv_pair *nv, int line, 
@@ -83,7 +82,6 @@ static int krb5_client_name_parser(struct nv_pair *nv, int line,
 		remote_conf_t *config);
 static int krb5_key_file_parser(struct nv_pair *nv, int line, 
 		remote_conf_t *config);
-#endif
 static int network_retry_time_parser(struct nv_pair *nv, int line, 
 		remote_conf_t *config);
 static int max_tries_per_record_parser(struct nv_pair *nv, int line, 
@@ -96,10 +94,11 @@ AP(network_failure)
 AP(disk_low)
 AP(disk_full)
 AP(disk_error)
-AP(remote_ending)
 AP(generic_error)
 AP(generic_warning)
 #undef AP
+static int remote_ending_action_parser(struct nv_pair *nv, int line,
+                remote_conf_t *config);
 static int sanity_check(remote_conf_t *config, const char *file);
 
 static const struct kw_pair keywords[] = 
@@ -115,12 +114,10 @@ static const struct kw_pair keywords[] =
   {"max_tries_per_record",   max_tries_per_record_parser,       0 },
   {"max_time_per_record",    max_time_per_record_parser,        0 },
   {"heartbeat_timeout",      heartbeat_timeout_parser,          0 },
-#ifdef USE_GSSAPI
-  {"enable_krb5",            enable_krb5_parser,                 0 },
-  {"krb5_principal",         krb5_principal_parser,              0 },
-  {"krb5_client_name",       krb5_client_name_parser,            0 },
-  {"krb5_key_file",          krb5_key_file_parser,               0 },
-#endif
+  {"enable_krb5",            enable_krb5_parser,                0 },
+  {"krb5_principal",         krb5_principal_parser,             0 },
+  {"krb5_client_name",       krb5_client_name_parser,           0 },
+  {"krb5_key_file",          krb5_key_file_parser,              0 },
   {"network_failure_action", network_failure_action_parser,	0 },
   {"disk_low_action",        disk_low_action_parser,		0 },
   {"disk_full_action",       disk_full_action_parser,		0 },
@@ -128,7 +125,7 @@ static const struct kw_pair keywords[] =
   {"remote_ending_action",   remote_ending_action_parser,	0 },
   {"generic_error_action",   generic_error_action_parser,	0 },
   {"generic_warning_action", generic_warning_action_parser,	0 },
-  { NULL,             NULL }
+  { NULL,                    NULL,                              0 }
 };
 
 static const struct nv_list transport_words[] =
@@ -140,7 +137,7 @@ static const struct nv_list transport_words[] =
 static const struct nv_list mode_words[] =
 {
   {"immediate",  M_IMMEDIATE },
-  {"forward",    M_STORE_AND_FORWARD },
+//  {"forward",    M_STORE_AND_FORWARD },
   { NULL,  0 }
 };
 
@@ -180,7 +177,7 @@ void clear_config(remote_conf_t *config)
 	config->remote_server = NULL;
 	config->port = 60;
 	config->local_port = 0;
-	config->port = T_TCP;
+	config->transport = T_TCP;
 	config->mode = M_IMMEDIATE;
 	config->queue_depth = 20;
 	config->format = F_MANAGED;
@@ -188,14 +185,7 @@ void clear_config(remote_conf_t *config)
 	config->network_retry_time = 1;
 	config->max_tries_per_record = 3;
 	config->max_time_per_record = 5;
-
 	config->heartbeat_timeout = 0;
-#ifdef USE_GSSAPI
-	config->enable_krb5 = 0;
-	config->krb5_principal = NULL;
-	config->krb5_client_name = NULL;
-	config->krb5_key_file = NULL;
-#endif
 
 #define IA(x,f) config->x##_action = f; config->x##_exe = NULL
 	IA(network_failure, FA_STOP);
@@ -206,6 +196,11 @@ void clear_config(remote_conf_t *config)
 	IA(generic_error, FA_SYSLOG);
 	IA(generic_warning, FA_SYSLOG);
 #undef IA
+
+	config->enable_krb5 = 0;
+	config->krb5_principal = NULL;
+	config->krb5_client_name = NULL;
+	config->krb5_key_file = NULL;
 }
 
 int load_config(remote_conf_t *config, const char *file)
@@ -404,31 +399,40 @@ static const struct kw_pair *kw_lookup(const char *val)
 	return &keywords[i];
 }
 
-static int check_exe_name(const char *val)
+static int check_exe_name(const char *val, int line)
 {
 	struct stat buf;
 
+	if (val == NULL) {
+		syslog(LOG_ERR, "Executable path needed for line %d", line);
+		return -1;
+	}
+
 	if (*val != '/') {
-		syslog(LOG_ERR, "Absolute path needed for %s", val);
+		syslog(LOG_ERR, "Absolute path needed for %s - line %d",
+			val, line);
 		return -1;
 	}
 
 	if (stat(val, &buf) < 0) {
-		syslog(LOG_ERR, "Unable to stat %s (%s)", val,
-			strerror(errno));
+		syslog(LOG_ERR, "Unable to stat %s (%s) - line %d", val,
+			strerror(errno), line);
 		return -1;
 	}
 	if (!S_ISREG(buf.st_mode)) {
-		syslog(LOG_ERR, "%s is not a regular file", val);
+		syslog(LOG_ERR, "%s is not a regular file - line %d", val,
+			line);
 		return -1;
 	}
 	if (buf.st_uid != 0) {
-		syslog(LOG_ERR, "%s is not owned by root", val);
+		syslog(LOG_ERR, "%s is not owned by root - line %d", val,
+			line);
 		return -1;
 	}
 	if ((buf.st_mode & (S_IRWXU|S_IRWXG|S_IWOTH)) !=
 			   (S_IRWXU|S_IRGRP|S_IXGRP)) {
-		syslog(LOG_ERR, "%s permissions should be 0750", val);
+		syslog(LOG_ERR, "%s permissions should be 0750 - line %d", val,
+			line);
 		return -1;
 	}
 	return 0;
@@ -444,7 +448,8 @@ static int server_parser(struct nv_pair *nv, int line,
 	return 0;
 }
 
-static int parse_uint (struct nv_pair *nv, int line, unsigned int *valp, unsigned int min, unsigned int max)
+static int parse_uint (const struct nv_pair *nv, int line, unsigned int *valp,
+		unsigned int min, unsigned int max)
 {
 	const char *ptr = nv->value;
 	unsigned int i;
@@ -487,12 +492,15 @@ static int parse_uint (struct nv_pair *nv, int line, unsigned int *valp, unsigne
 
 static int port_parser(struct nv_pair *nv, int line, remote_conf_t *config)
 {
-	return parse_uint (nv, line, &(config->port), 0, INT_MAX);
+	return parse_uint (nv, line, &(config->port), 0, 65535);
 }
 
-static int local_port_parser(struct nv_pair *nv, int line, remote_conf_t *config)
+static int local_port_parser(struct nv_pair *nv, int line,
+		remote_conf_t *config)
 {
-	return parse_uint (nv, line, &(config->local_port), 0, INT_MAX);
+	if ((strcasecmp(nv->value, "any") == 0))
+		return 0;	// The default is 0, which means any port
+	return parse_uint (nv, line, &(config->local_port), 0, 65535);
 }
 
 static int transport_parser(struct nv_pair *nv, int line, remote_conf_t *config)
@@ -500,7 +508,7 @@ static int transport_parser(struct nv_pair *nv, int line, remote_conf_t *config)
 	int i;
 	for (i=0; transport_words[i].name != NULL; i++) {
 		if (strcasecmp(nv->value, transport_words[i].name) == 0) {
-			config->mode = transport_words[i].option;
+			config->transport = transport_words[i].option;
 			return 0;
 		}
 	}
@@ -522,7 +530,7 @@ static int mode_parser(struct nv_pair *nv, int line, remote_conf_t *config)
 }
 
 static int depth_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
 	return parse_uint (nv, line, &(config->queue_depth), 1, INT_MAX);
 }
@@ -537,8 +545,8 @@ static int action_parser(struct nv_pair *nv, int line,
 			return 0;
 		} else if (i == FA_EXEC) {
 			if (strncasecmp(fail_action_words[i].name,
-							nv->value, 4) == 0){
-				if (check_exe_name(nv->option))
+							nv->value, 4) == 0) {
+				if (check_exe_name(nv->option, line))
 					return 1;
 				*exep = strdup(nv->option);
 				*actp = FA_EXEC;
@@ -554,20 +562,30 @@ static int action_parser(struct nv_pair *nv, int line,
 static int x##_action_parser(struct nv_pair *nv, int line, \
 	remote_conf_t *config) \
 { \
-	return action_parser (nv, line, &(config->x##_action), &(config->x##_exe)); \
+	return action_parser(nv,line,&(config->x##_action),&(config->x##_exe));\
 } \
 
 AP(network_failure)
 AP(disk_low)
 AP(disk_full)
 AP(disk_error)
-AP(remote_ending)
 AP(generic_error)
 AP(generic_warning)
 #undef AP
 
+static int remote_ending_action_parser(struct nv_pair *nv, int line,
+                remote_conf_t *config)
+{
+	if (strcasecmp(nv->value, "reconnect") == 0) {
+		config->remote_ending_action = FA_RECONNECT;
+		return 0;
+	}
+	return action_parser(nv, line, &config->remote_ending_action,
+			&config->remote_ending_exe);
+}
+
 static int format_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
 	int i;
 	for (i=0; format_words[i].name != NULL; i++) {
@@ -581,33 +599,38 @@ static int format_parser(struct nv_pair *nv, int line,
 }
 
 static int network_retry_time_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
-	return parse_uint (nv, line, &(config->network_retry_time), 1, INT_MAX);
+	return parse_uint(nv, line, &config->network_retry_time, 1, INT_MAX);
 }
 
 static int max_tries_per_record_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
-	return parse_uint (nv, line, &(config->max_tries_per_record), 1, INT_MAX);
+	return parse_uint(nv, line, &config->max_tries_per_record, 1, INT_MAX);
 }
 
 static int max_time_per_record_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
-	return parse_uint (nv, line, &(config->max_time_per_record), 1, INT_MAX);
+	return parse_uint(nv, line, &(config->max_time_per_record), 1, INT_MAX);
 }
 
 static int heartbeat_timeout_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
 	return parse_uint (nv, line, &(config->heartbeat_timeout), 0, INT_MAX);
 }
 
-#ifdef USE_GSSAPI
 static int enable_krb5_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
+#ifndef USE_GSSAPI
+	syslog(LOG_INFO,
+		"GSSAPI support is not enabled, ignoring value at line %d",
+		line);
+	return 0;
+#else
 	unsigned long i;
 
 	for (i=0; enable_krb5_values[i].name != NULL; i++) {
@@ -618,44 +641,56 @@ static int enable_krb5_parser(struct nv_pair *nv, int line,
 	}
 	syslog(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
+#endif
 }
 
 static int krb5_principal_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
-	const char *ptr = nv->value;
-
+#ifndef USE_GSSAPI
+	syslog(LOG_INFO,
+		"GSSAPI support is not enabled, ignoring value at line %d",
+		line);
+#else
 	if (config->krb5_principal)
 		free ((char *)config->krb5_principal);
 
-	config->krb5_principal = strdup(ptr);
+	config->krb5_principal = strdup(nv->value);
+#endif
 	return 0;
 }
 
 static int krb5_client_name_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
-	const char *ptr = nv->value;
-
+#ifndef USE_GSSAPI
+	syslog(LOG_INFO,
+		"GSSAPI support is not enabled, ignoring value at line %d",
+		line);
+#else
 	if (config->krb5_client_name)
 		free ((char *)config->krb5_client_name);
 
-	config->krb5_client_name = strdup(ptr);
+	config->krb5_client_name = strdup(nv->value);
+#endif
 	return 0;
 }
 
 static int krb5_key_file_parser(struct nv_pair *nv, int line,
-	remote_conf_t *config)
+		remote_conf_t *config)
 {
-	const char *ptr = nv->value;
-
+#ifndef USE_GSSAPI
+        syslog(LOG_INFO,
+                "GSSAPI support is not enabled, ignoring value at line %d",
+                line);
+#else
 	if (config->krb5_key_file)
 		free ((char *)config->krb5_key_file);
 
-	config->krb5_key_file = strdup(ptr);
+	config->krb5_key_file = strdup(nv->value);
+#endif
 	return 0;
 }
-#endif
 
 /*
  * This function is where we do the integrated check of the audispd config
@@ -675,5 +710,15 @@ static int sanity_check(remote_conf_t *config, const char *file)
 void free_config(remote_conf_t *config)
 {
 	free((void *)config->remote_server);
+	free((void *)config->network_failure_exe);
+	free((void *)config->disk_low_exe);
+	free((void *)config->disk_full_exe);
+	free((void *)config->disk_error_exe);
+	free((void *)config->remote_ending_exe);
+	free((void *)config->generic_error_exe);
+	free((void *)config->generic_warning_exe);
+	free((void *)config->krb5_principal);
+	free((void *)config->krb5_client_name);
+	free((void *)config->krb5_key_file);
 }
 
