@@ -68,6 +68,9 @@ _S(AUDIT_USER_CMD,                   "USER_CMD"                      )
 _S(AUDIT_USER_TTY,                   "USER_TTY"                      )
 _S(AUDIT_CHUSER_ID,                  "CHUSER_ID"                     )
 _S(AUDIT_GRP_AUTH,                   "GRP_AUTH"                      )
+_S(AUDIT_SYSTEM_BOOT,                "SYSTEM_BOOT"                   )
+_S(AUDIT_SYSTEM_SHUTDOWN,            "SYSTEM_SHUTDOWN"               )
+_S(AUDIT_SYSTEM_RUNLEVEL,            "SYSTEM_RUNLEVEL"               )
 _S(AUDIT_DAEMON_START,               "DAEMON_START"                  )
 _S(AUDIT_DAEMON_END,                 "DAEMON_END"                    )
 _S(AUDIT_DAEMON_ABORT,               "DAEMON_ABORT"                  )
