@@ -283,4 +283,10 @@ _S(318, "eventfd")
 _S(319, "timerfd_create")
 _S(320, "timerfd_settime")
 _S(321, "timerfd_gettime")
+_S(322, "signalfd4")
+_S(323, "eventfd2")
+_S(324, "inotify_init1")
+_S(325, "pipe2")
+_S(326, "dup3")
+_S(327, "epoll_create1")
 
