@@ -177,6 +177,9 @@ static void print_title_summary(void)
 			printf("total  file\n");
 			printf("===========================\n");
 			break;
+		case RPT_TTY:
+			UNIMPLEMENTED;
+			break;
 		default:
 			break;
 	}
@@ -222,11 +225,11 @@ static void print_title_detailed(void)
 		case RPT_ACCT_MOD:
 			printf("Account Modifications Report\n");
 			printf(
-			  "============================================\n");
+			 "=================================================\n");
 			printf(
-			  "# date time auid addr term exe success event\n");
+			 "# date time auid addr term exe acct success event\n");
 			printf(
-			  "============================================\n");
+			 "=================================================\n");
 			break;
 		case RPT_TIME:
 			printf("Log Time Range Report\n");
@@ -399,6 +402,20 @@ static void print_title_detailed(void)
 				printf("====================\n");
 			}
 			break;
+		case RPT_TTY:
+			if (report_detail == D_DETAILED) {
+			    printf("TTY Report\n");
+			    printf(
+			"===============================================\n");
+			    printf(
+			"# date time event auid term sess comm data\n");
+			    printf(
+			"===============================================\n");
+			} else {
+				printf("Specific TTY Report\n");
+				printf("====================\n");
+			}
+			break;
 		default:
 			break;
 	}
@@ -473,11 +490,12 @@ void print_per_event_item(llist *l)
 			break;
 		case RPT_ACCT_MOD:
 			// who, addr, terminal, exe, success, event
-			printf("%s %s %s %s %s %lu\n",
+			printf("%s %s %s %s %s %s %lu\n",
 				aulookup_uid(l->s.loginuid, name, sizeof(name)),
 				l->s.hostname ? l->s.hostname : "?",
 				l->s.terminal ? l->s.terminal : "?",
 				l->s.exe ? l->s.exe : "?",
+				l->s.acct ? l->s.acct : "?",
 				aulookup_success(l->s.success),
 				l->e.serial);
 			break;
@@ -596,6 +614,25 @@ void print_per_event_item(llist *l)
 				aulookup_uid(l->s.loginuid, name, sizeof(name)),
 				l->e.serial);
 			break;
+		case RPT_TTY: {
+			char *ch, *ptr = strstr(l->head->message, "data=");
+			if (!ptr)
+				break;
+			ptr += 5;
+			ch = strrchr(ptr, ' ');
+			if (ch)
+				*ch = 0;
+			// event who term sess data
+			printf("%lu %s %s %u %s ",
+				l->e.serial,
+				aulookup_uid(l->s.loginuid, name, sizeof(name)),
+				l->s.terminal ? l->s.terminal : "?",
+				l->s.session_id,
+				l->s.comm ? l->s.comm: "?");
+			print_tty_data(ptr);
+			printf("\n");
+			}
+			break;
 		default:
 			break;
 	}
